.Dd Jun 16, 2009
.Dt dorrit 8
.Sh NAME
.Nm Dorrit
.Nd Minimal KQueue-Based HTTP/1.1 Server
.Sh SYNOPSIS
.Nm dorrit -r Ao server-root Ac Oo -m Ao instances Ac -i Ao interface Ac -p Ao port Ac -e Ao root-resource Ac -t Ao read-timeout Ac -w Ao write-timeout Ac -u Ao user Ac -g Ao group Ac -q Ao backlog Ac -c -x Oc
.Sh DESCRIPTION
Dorrit is a lightweight, partially-compliant HTTP/1.1 server, which can
only serve static content.
.Pp
.Ss WHAT DORRIT DOES
.Pp
.Bl -bullet
.It
Dorrit can service both IPv4 and IPv6 connections.
.It
Dorrit understands the HEAD and GET request methods, only.
.It
Dorrit is configured almost entirely by command-line options, with the
single exception of one user-modifiable configuration file to define the
mappings of filename suffixes to MIME types.
.It
Dorrit supports HTTP/1.1 persistent connections, with user-modifiable
timeouts for limiting the time it takes to both read requests and write
responses.  Dorrit ignores the "Keep-Alive" request header.
.It
Dorrit supports name-based virtual hosting, dictated by the value of the
"Host" request header.  This allows one physical host to serve up data for
more than one host name.
.It
If a client sends an "Accept-Encoding" header indicating it is able to
receive gzip-compressed content, and dorrit can find a version of the
requested resource with a ".gz" suffix, it will send the compressed
resource to the client instead of the uncompressed resource.
.El
.Pp
.Ss WHAT DORRIT DOES NOT DO
.Bl -bullet
.It
Dorrit does not accept HTTPS connections.
.It
Dorrit does not perform access authentication.
.It
Dorrit does not perform transaction logging.
.It
Dorrit does not restart "gracefully".  After receiving a SIGTERM,
established connections will be dropped without allowing transactions in
progress to complete.
.It
Dorrit Does not perform content negotiation, beyond recognizing the
"Accept-Encoding", "If-Modified-Since", and "If-Unmodified-Since" request
headers.  The only accept-encoding supported is gzip.
.It
Dorrit does not provide any response headers useful for caching server
responses beyond the "Date" and "Last-Modified" headers.
.It
Dorrit will not generate directory listings for clients.
.It
dorrit does not support the ~user notation to provide access to resources in
user directories, but this may be simulated with symbolic links.
.El
.Ss SYSTEM CONFIGURATION
Dorrit is a kqueue(2)-based multiplexer.  By default, FreeBSD limits each
process to having a maximum of 64 simultaneously open descriptors.  This
places a limit on the number of connections dorrit may multiplex.  The
sysctl settings below, allow for a generous number of connections.  These
values can be set at the command-line with the "sysctl" utility.  It is
recommended that they be enabled permanently by placing these lines in
/etc/sysctl.conf on the host machine.
.Bd -literal -offset left
kern.maxfiles=16384
kern.maxfilesperproc=16384
kern.kq_calloutmax=65536
kern.ipc.nmbclusters=25600
kern.ipc.maxsockets=25600
.Ed
.Pp
Dorrit will attempt to set the accf_http accept filter on the listening
socket, to increase performance.  For this to succeed, you will need to
load the module before starting dorrit with:
.Bd -literal -offset left
kldload accf_http
.Ed
.Pp
To load the module automatically at system boot-up, add the following line
to /boot/loader.conf:
.Bd -literal -offset left
accf_http_load="YES"
.Ed
.Ss DORRIT CONFIGURATION
Dorrit writes its pid into /var/run/dorrit.pid, if it can (ie., it is started
as root), and may be stopped with a SIGTERM.  A rc.d script is provided and
installed in /usr/local/etc/rc.d/.  Add the following lines to
/etc/rc.conf to start dorrit on system boot-up.  Replace the items in
brackets with values appropriate for your system.  These are the minimal
set of options one should start with.  These and all of the other available
options are described in full at the end of this manual page.
.Bd -literal -offset left
scgid_enable="NO"
drood_enable="NO"
dorrit_enable="YES"
dorrit_flags="-r <server-root> -u <user> -g <group> -m <instances>"
.Ed
.Pp
Once your system is configured you may start, stop, or restart dorrit, or
determine if it is running with the following commands:
.Bd -literal -offset left
/usr/local/etc/rc.d/dorrit start
/usr/local/etc/rc.d/dorrit stop
/usr/local/etc/rc.d/dorrit restart
/usr/local/etc/rc.d/dorrit status
.Ed
.Pp
If you do not want dorrit started on system start, then set
.Bd -literal -offset left
dorrit_enable="NO"
.Ed
.Pp
and use the following commands:
.Bd -literal -offset left
/usr/local/etc/rc.d/dorrit forcestart
/usr/local/etc/rc.d/dorrit forcestop
/usr/local/etc/rc.d/dorrit forcerestart
/usr/local/etc/rc.d/dorrit forcestatus
.Ed
.Pp
.Ss GZIP CONTENT ENCODING
To enable gzip content-encoding, the webmaster must manually gzip the
resources he or she would like to make available in that encoding.  The
original resource should not be replaced by the gzipped version, but should
also be available in the same directory for clients which cannot handle the
encoding.  The gzipped version must have the ".gz" filename suffix.  If a
client indicates it is acceptable for a requested resource to be gzipped,
then dorrit will attempt to send the gzipped version of the resource.  If
this attempt fails, dorrit will attempt to send the non-compressed version
of the requested resource.
.Pp
Do not confuse gzip content-encoding with naturally gzipped resources, such
as tar.gz files.  An explicit request for a gzipped resource will be cause
the resource to be transferred to the client, as expected.  With gzip
content-encoding, the client requests a resource, such as an *.html file,
indicating it can accept that resource gzipped, to save bandwidth and
decrease transfer time.  Dorrit will look for a gzipped version of the
requested resource and send it instead, if it exists.  All major browsers
support gzip content-encoding, so creating gzipped versions of your files
is a useful thing to do, to both speed up transfers and lower the server's
bandwidth consumption.
.Pp
Note that dorrit does not compare the modification times of the uncompressed
and compressed resources.  It will always send the compressed resource if
it exists, is acceptable to the client, and dorrit has permission to read
it.  One can keep Makefiles in one's html directories to compress all the
resources whose compressed versions are out of date, with gzip -k.  As long
as the Makefiles are not readable by the user or group dorrit is running as,
client requests for them will fail.
.Ss MAPPING FILENAME SUFFIXES TO MIME TYPES
These are the mappings from filename suffixes to MIME types which dorrit
recognizes out-of-the-box.  They are listed in types.tab in
/usr/local/etc/drood/.  Any file which cannot be identified by these
mappings is identified by the server as "text/plain; charset=utf-8".  To
add a mapping, simply add a line to types.tab.  Each line must consist of
the filename suffix to be mapped, followed by one or more tab characters,
followed by the MIME type, followed by a newline.  Blank lines are ignored,
after the first line.  The first line must not be blank.  If dorrit finds
empty fields in types.tab it will refuse to run.  If you modify types.tab,
you must restart dorrit for the changes to take effect.
.Bl -column -offset left "jpeg" "application/x-shockwave-flash"
.It Sy Suffix Ta Sy MIME Type
.It html Ta text/html; charset=utf-8
.It htm  Ta text/html; charset=utf-8
.It xml  Ta text/xml; charset-utf-8
.It txt  Ta text/plain; charset=utf-8
.It css  Ta text/css
.It ico  Ta image/vnd.microsoft.icon
.It png  Ta image/png
.It jpg  Ta image/jpeg
.It jpeg Ta image/jpeg
.It tiff Ta image/tiff
.It tif  Ta image/tiff
.It gif  Ta image/gif
.It tgz  Ta application/x-compressed
.It zip  Ta application/zip
.It ps   Ta application/postscript
.It eps  Ta application/postscript
.It gz   Ta application/x-gzip
.It tgz  Ta application/x-compressed
.It pdf  Ta application/pdf
.It swf  Ta application/x-shockwave-flash
.It js   Ta application/x-javascript
.It mov  Ta video/quicktime
.It qt   Ta video/quicktime
.It mpg  Ta video/mpeg
.It mp2  Ta video/mpeg
.It mpa  Ta video/mpeg
.It mpe  Ta video/mpeg
.It avi  Ta video/x-msvideo
.It asf  Ta video/x-ms-asf
.It asx  Ta video/x-ms-asf
.It asr  Ta video/x-ms-asf
.It flv  Ta video/x-flv
.It ra   Ta audio/x-pn-realaudio
.It ram  Ta audio/x-pn-realaudio
.It wav  Ta audio/x-wav
.It mp3  Ta audio/mpeg
.El
.Pp
.Ss COMMAND-LINE OPTIONS
The following options are recognized.  Only the -r option is required, but
you will probably want to include at least the -u, and -g options when you
are starting the server for production use.  For optimal performance, employ
the -m option, and use the -q and -t options to match your load levels.
.Bl -tag -width "-r"
.It -r
The -r option is mandatory and specifies the server root directory, which
dorrit will make its current working directory.
.Pp
The resources served-up by dorrit are determined by the "Host" header of
requests, thus implementing name-based virtual hosting.  For every hostname
you wish to be recognized by the server, an identically-named subdirectory
of the server root directory must be created.  The files served up by each
virtual host are placed into the identically-named subdirectory.  Any or
all of the subdirectories may be symbolic links.  One subdirectory must be
named "default".  This is the directory which will be used to fulfill
requests when no "Host" header is supplied by a client (eg., HTTP/1.0
requests).  The "default" directory can and should be a symbolic link.  If
you would like a directory to be used for more than one hostname, you must
create links for the extra hostnames.
.Pp
For example, I could have, in my server root directory, the following
subdirectories.  Note the symlink for the loopback addresses.  Without it,
dorrit will not be able to find resources for requests from loopback
connections, if clients use the address in URIs, instead of the local
hostname.  Note the link for the IP address of the server, to allow clients
to connect using URIs containing the server's IP address instead of its
hostname.  Note that if you are using IPv6, you will need to create links
which include the surrounding square brackets which are used to identify
IPv6 addresses in URIs (rfc3986).
.Bd -literal -offset left
mammothcheese.ca     -> actual subdirectory
www.mammothcheese.ca -> symlink to mammothcheese.ca
default              -> symlink to mammothcheese.ca
127.0.0.1            -> symlink to mammothcheese.ca
[::1]                -> symlink to mammothcheese.ca (IPv6 Loopback)
99.250.61.49         -> symlink to mammothcheese.ca
.Ed
.Pp
Dorrit will reject any request whose host header contains a virgule "/", or
starts with "." in order to prevent clients from accessing resources
outside of the virtual host subdirectories.  Note this does not prevent you
from symlinking a directory outside of the server root directory hierarchy
into the server root directory, if the server is not running chrooted (the
-c option).
.Pp
Symbolic links can be used to provide access to user websites.  For
example, if I wished to expose the personal website of user "ella" on
mammothcheese.ca to dorrit, I could create a symbolic link inside the
mammothcheese.ca directory named "ella" pointing to her ~/public_html
directory.  This would allow access via URIs such as
http://www.mammothcheese.ca/ella/.  Note that dorrit will recognize a
request ending in a virgule "/" as a request for the root resource in the
directory specified by the request.
.Pp
Remember to set the ownership and permissions of the directories and their
contents according to the values you give for the -u and -g options.
Static resources must be set readable by the user or group dorrit will be
running as.  Directories must similarly be searchable.
.Pp
.It -e
The root resource of all virtual hosts defaults "index.html", which itself
may be a symbolic link.  This may be changed with the -e option.  Note this
changes the name of the root resource of all virtual hosts served-up by
dorrit.  The value specified must not begin with a virgule "/".  If a
request is received ending with a virgule "/", that request is interpreted
as a request for the root resource residing in the directory specified by
the request.
.Pp
If the root resource is a symbolic link and you wish to make use of gzip
content-encoding for that resource when requested via the symbolic link,
make a symbolic link to the compressed version of the root resource as
well.  For example, if you have index.html linked to another_file.html,
create a compressed version of another_file.html if one does not already
exist, with "gzip -k another_file.html".  Then create a link for it with
"ln -s another_file.html.gz index.html.gz"  Remember to set the file
permissions accordingly for the values you have used for the -u and -g
options.
.It -c
The -c option, if present, makes dorrit chroot(2) to the server root
directory.
.It -p
The -p option specifies the port to listen on.  This defaults to 80 if not
specified.
.It -i
By default, dorrit accepts connections on all interfaces it can find capable
of IPv4 or IPv6.  The -i option, when present, overrides this behavior, by
limiting dorrit to accepting connections from a specified interface only.
The option accepts the IP address of the desired interface as an argument.
The address must be expressed in the presentation format for either IPv4 or
IPv6.
.Pp
The intent of this option is to allow the administrator to prevent the
outside world from accessing a server during testing by making dorrit bind
only to one interface (eg., the loopback address).
.It -u
.It -g
Dorrit must be started as root in order to bind to port 80, but it tries to
setuid to user "nobody" and setgid to group "nobody" after binding the
listening socket.  If it is unsuccessful in either attempt, the server will
exit.  The -u and the -g options may be used to specify a different user
and group.  If you use the -p option to make dorrit listen on a port higher
than port 1023, then dorrit can be started as any user, but it will not be
able to change to user and group "nobody".  In this case, you must provide
the -u and -g options to specify the user and group that dorrit will begin
running as and stay running as, or the server will exit.
.It -t
When reading a request header from a client, dorrit drops the connection if
it cannot read the complete request header within twenty seconds.  The -t
option, if present, specifies a different read timeout value, in seconds.
Clients typically keep a connection to a server open to aid responsiveness
if the user subsequently requests more resources from the same server, and
rely on the server to timeout idle connections.  The value must be in the
range 1-60 seconds.  It is important this value take into consideration the
latency of dorrit on your system under load, and the latency of your network
interfaces, and the possible latency of clients behind firewalls on busy
networks.  If the server starts dropping connections under load, and you
haven't run out of resources, you may have the read timeout set too low.
.It -w
When dorrit is in the process of writing a requested resource to a client,
the connection will be dropped if 60 minutes has elapsed since it started
writing the resource.  This value may be changed with the -w option, and must
be in range of 0-120 minutes.  Setting this value to 0, disables the write
timeout.  In this case, the client may take as long as it likes to read a
requested resource.  Note that persistent connections may exist for longer
periods than this value specifies, but each individual resource requested
over such a connection must be read by the client within this number of
minutes, or the connection will be dropped.
.It -q
The -q option, if present, specifies the backlog of client connections
queued by the OS kernel for the server to subsequently service.  This value
defaults to 1000.  Note that the OS kernel actually uses a queue of 1.5
times the size of the specified value.  Connections arriving when the queue
is full are dropped by the kernel.  This value should be set to a generous
value.  If the system starts rejecting connections under load, and you're
sure you haven't run out of resources, try increasing the size of the
backlog queue.
.It -x
The -x option, if present, prevents dorrit from becoming a daemon.  It will
then run in the foreground of the terminal where it was started, and may be
stopped with signals (ie., Control-C).  The server also will not write its
pid to /var/run/dorrit.pid when the -x option is used.
.It -m
The -m option specifies how many instances of dorrit should be run, and
defaults to 1.  This option is intended for use on multiple-core or
multiple-processor machines, where running a separate instance of dorrit for
each core/processor will drastically increase the number of requests per
second dorrit can process.
.El
.Sh AUTHORS
.An James Bailie Aq jimmy@mammothcheese.ca
.br
http://www.mammothcheese.ca
