.Dd Jan 2, 2009
.Dt scgid 8
.Sh NAME
.Nm scgid
.Nd SCGI Server Stub
.Sh SYNOPSIS
.Nm scgid -l Ao responder Ac Oo -r Ao root dir Ac -n Ao min-servers Ac -m Ao max-servers Ac -i Ao interface Ac -p Ao port Ac -u Ao user Ac -g Ao group Ac -b Ao max-body Ac -x Oc
.Sh DESCRIPTION
Scgid allows programmers to use a filter, written in any language, as a
persistent SCGI server.
.Pp
Starting-up and shutting-down processes for traditional CGI programs
becomes a performance bottleneck on busy HTTP servers.  The Simple Common
Gateway Interface (SCGI) protocol specifies a simple means by which an HTTP
server can forward a request to a persistent application to generate the
response.  Process start-up and shut-down costs are eliminated for each
request, resulting in significantly better performance over traditional
CGI.
.Pp
Scgid is a multi-process SCGI server which communicates with HTTP servers
via the SCGI protocol, but which delegates the task of generating client
responses to another user-specified program.  This program may be written
in the language of choice of the programmer.  The documentation of the -l
option below, explains the structure of responder programs.
.Pp
Scgid writes its pid into /var/run/scgid.pid, if it can (ie., it is started
as root), and may be stopped with a SIGTERM.  A rc.d script is provided and
installed in /usr/local/etc/rc.d/.  Add the following lines to /etc/rc.conf
to start scgid on system boot-up.  Replace the items in brackets with
values appropriate for your system, and/or add or remove options as suits
your system.  The available options are described in full at the end of
this manual page.
.Bd -literal -offset left
scgid_enable="YES"
scgid_flags="-l <responder> -r <root dir> -u <user> -g <group>"
.Ed
.Pp
Once your /etc/rc.conf is configured you may start, stop, or restart scgid,
or determine if it is running with the following commands:  
.Bd -literal -offset left
/usr/local/etc/rc.d/scgid start
/usr/local/etc/rc.d/scgid stop
/usr/local/etc/rc.d/scgid restart
/usr/local/etc/rc.d/scgid status
.Ed
.Pp
If you do not want scgid started on system start, then set
.Bd -literal -offset left
scgid_enable="NO"
.Ed
.Pp
and use the following commands:
.Bd -literal -offset left
/usr/local/etc/rc.d/scgid forcestart
/usr/local/etc/rc.d/scgid forcestop
/usr/local/etc/rc.d/scgid forcerestart
/usr/local/etc/rc.d/scgid forcestatus
.Ed
.Pp
If the server encounters an error reading or writing data to or from
a client, or it runs out of memory, the particular connection which
generated the error will be dropped.  Errors are logged via syslog(3), and
may be found in /var/log/messages.
.Pp
The following arguments are recognized.  The -l option is mandatory.  The
rest are optional.
.Bl -tag -width "-i"
.It -l
The mandatory -l option specifies the filter program to handle client
requests, called the responder.  The executable file specified by this
option must either reside in a directory listed in the PATH environment
variable, or a fully-qualified filename must be provided.  If the server
encounters a problem forking this file, an error message will be logged via
syslog.
.Pp
Responder programs must be written as infinite loops.  In each iteration of
the loop, a responder must read a complete client request from its standard
input, before generating the response on its standard output.  When scgid
is shut down, the responder will read EOF from its standard input when it
attempts to read a new request and must exit in response.  The responder's
standard error is connected to /dev/null when the responder is running, and
so not usable.  Error messages should be sent as responses to the client,
or logged via syslog(3) or logger(1).
.Pp
Client requests consist of two parts, a header and a body.  The header
contains all the SCGI environment variables listed with their values, one
pair per line, separated by a colon character ":", with each line
terminated by a newline.  Any whitespace adjacent to the colon is
significant and part of the variable or its value.  The header is
terminated by a blank line consisting of a single newline character.  The
first environment variable pair of the header will be the CONTENT_LENGTH
variable and its value.  The value is the length, in bytes, of the body.
Immediately following the blank line terminating the header is the body.
The body has a single newline character appended to it which is not counted
in the value of the CONTENT_LENGTH variable.  It is added by scgid, to
ensure that the body is always newline-terminated.  The responder should
remove the terminating newline from the body data before processing it
further.  Immediately following this newline is a line of text containing
the seven characters, __END__, followed by a newline character.  This is
the request terminator.  Responders must not attempt to read beyond the
request terminator until they have output a complete response and are ready
to process another request.  Responders which don't care about the body, or
which expect an empty body, may continue to read lines past the header
until they encounter the __END__ marker.  The complete request must be
read.
.Pp
After reading a request, the responder generates an appropriate CGI
response message.  After sending the response to its standard output, the
responder must send a single newline character, followed by the seven
characters, __END__, followed by another single newline character to its
standard output.  These 9 characters are the response terminator and will
be removed from the response by scgid.
.Pp
Drood requires that the resource specified in the client's request URL
actually exist in the filesystem with the appropriate permissions, but
because the SCGI protocol is a means of forwarding a request to another
server, drood does not do anything with this entity.  SCGI programs do not
have to do anything with it either.  It may be an empty file.  Its path
relative to drood's root directory will be specified by the SCRIPT_NAME
environment variable.  Responders may use this value as another
discriminator in addition to the CGI parameters present in QUERY_STRING or
the request body.
.It -r
The -r option specifies the server root directory.  The server will
chdir(2) here.  Setting the value of this option to the web server root
directory will usually mean the paths in SCRIPT_NAME can used as is,
because they are usually specified as absolute paths, but as if the web
server root were the filesystem root.
.It -b
The -b option specifies the maximum size in bytes of request bodies that
scgid will accept.  If a client attempts to send a request body larger than
the amount specified by this option, the connection will be dropped.  If
not specified, this value defaults to 50000.  Note that scgid reads all the
request body data into memory before writing it to the responder, so that
setting this option is a way of limiting memory usage.
.It -p
The -p option specifies the port to listen on.  This defaults to 4000 if
not specified.
.It -i
By default, scgid accepts connections on all interfaces it can find capable
of IPv4 or IPv6.  The -i option, when present, overrides this behavior, by
limiting scgid to accepting connections from a specified interface only.
The option accepts the IP address of the desired interface as an argument.
The address must be expressed in the presentation format for either IPv4 or
IPv6.  The intent of this option is to allow the administrator to prevent
the outside world from accessing a server during testing by making scgid
bind only to one interface (eg., the loopback address).
.It -m
.It -n
Scgid uses a multi-process model, with one master process forking or
killing slave processes in response to client activity.  Only the slave
processes service client requests.  The -n options specifies the number of
idle slaves the master will attempt to keep ready at all times.  The -m
option specifies the maximum number of slaves which may be running at any
time.  The value specified must be equal to or greater than the value
specified for -n.  The default value for -n is 3 and the default value for
-m is 25.
.It -u
.It -g
Scgid tries to setuid to user "nobody" and setgid to group "nobody" after
binding the listening socket.  If it is unsuccessful, the server will exit.
To be able to change users, scgid must be started as root.  The -u and the
-g options may be used to specify a different user and group.
.It -x
The -x option, if present, prevents scgid from becoming a daemon.  It will
then run in the foreground of the terminal where it was started, and may be
stopped with signals (ie., Control-C).  The values of -m and -n will be
ignored, and only one iterative server will be run.  The server also will
not write its pid to /var/run/scgid.pid when the -x option is used.
.El
.Sh AUTHORS
.An James Bailie Aq jimmy@mammothcheese.ca
.br
http://www.mammothcheese.ca
